@@ -23,37 +23,37 @@ class TestUrljoin(unittest.TestCase):
 
         cf = CollectionFetcher({}, None)
 
-        self.assertEquals("keep:99999999999999999999999999999991+99/hw.py",
+        self.assertEqual("keep:99999999999999999999999999999991+99/hw.py",
                           cf.urljoin("keep:99999999999999999999999999999991+99", "hw.py"))
 
-        self.assertEquals("keep:99999999999999999999999999999991+99/hw.py",
+        self.assertEqual("keep:99999999999999999999999999999991+99/hw.py",
                           cf.urljoin("keep:99999999999999999999999999999991+99/", "hw.py"))
 
-        self.assertEquals("keep:99999999999999999999999999999991+99/hw.py#main",
+        self.assertEqual("keep:99999999999999999999999999999991+99/hw.py#main",
                           cf.urljoin("keep:99999999999999999999999999999991+99", "hw.py#main"))
 
-        self.assertEquals("keep:99999999999999999999999999999991+99/hw.py#main",
+        self.assertEqual("keep:99999999999999999999999999999991+99/hw.py#main",
                           cf.urljoin("keep:99999999999999999999999999999991+99/hw.py", "#main"))
 
-        self.assertEquals("keep:99999999999999999999999999999991+99/dir/hw.py#main",
+        self.assertEqual("keep:99999999999999999999999999999991+99/dir/hw.py#main",
                           cf.urljoin("keep:99999999999999999999999999999991+99/dir/hw.py", "#main"))
 
-        self.assertEquals("keep:99999999999999999999999999999991+99/dir/wh.py",
+        self.assertEqual("keep:99999999999999999999999999999991+99/dir/wh.py",
                           cf.urljoin("keep:99999999999999999999999999999991+99/dir/hw.py", "wh.py"))
 
-        self.assertEquals("keep:99999999999999999999999999999991+99/wh.py",
+        self.assertEqual("keep:99999999999999999999999999999991+99/wh.py",
                           cf.urljoin("keep:99999999999999999999999999999991+99/dir/hw.py", "/wh.py"))
 
-        self.assertEquals("keep:99999999999999999999999999999991+99/wh.py#main",
+        self.assertEqual("keep:99999999999999999999999999999991+99/wh.py#main",
                           cf.urljoin("keep:99999999999999999999999999999991+99/dir/hw.py", "/wh.py#main"))
 
-        self.assertEquals("keep:99999999999999999999999999999991+99/wh.py",
+        self.assertEqual("keep:99999999999999999999999999999991+99/wh.py",
                           cf.urljoin("keep:99999999999999999999999999999991+99/hw.py#main", "wh.py"))
 
-        self.assertEquals("keep:99999999999999999999999999999992+99",
+        self.assertEqual("keep:99999999999999999999999999999992+99",
                           cf.urljoin("keep:99999999999999999999999999999991+99", "keep:99999999999999999999999999999992+99"))
 
-        self.assertEquals("keep:99999999999999999999999999999991+99/dir/wh.py",
+        self.assertEqual("keep:99999999999999999999999999999991+99/dir/wh.py",
                           cf.urljoin("keep:99999999999999999999999999999991+99/dir/", "wh.py"))
 
     def test_resolver(self):
